/*
 * NBodySimulation.cpp
 *
 *  Created on: Feb 21, 2009
 *      Author: sri
 */

#include "NBodySimulation.h"

void NBodySimulation::calculateCM(){
	static list<Point>::iterator * iterate;

	cm_x=cm_y=0.0f;
	if (isLeafNode()){
		mass=data.getNumPoints();
		iterate=data.getBegin();
		for(;iterate!=data.getEnd();iterate++){
			cm_x+=iterate->x;
			cm_y+=iterate->y;
		}
		cm_x/=(float)mass;
		cm_y/=(float)mass;
	}else{
		mass=0;
		for (unsigned int i=0;i<4;i++){
			children[i]->calculateCM();
			mass+=children[i]->mass;
			cm_x+=children[i]->mass*children[i]->cm_x;
			cm_y+=children[i]->mass*children[i]->cm_y;
		}
		cm_x/=(float)mass;
		cm_y/=(float)mass;
	}
}
void NBodySimulation::gatherForces(Point& p){
	static float distance,distance2;
	static list<Point>::iterator iterate;

	distance2=(cm_x-p.x)*(cm_x-p.x) + (cm_y-p.y)*(cm_y-p.y);
	distance=sqrt(distance2);

	if (distance>SIMULATION_ACCURACY && distance>0.001){
		p.fx+=(mass*(cm_x-p.x)/(distance2*distance));
		p.fy+=(mass*(cm_y-p.y)/(distance2*distance));
	}else if (!isLeafNode()){
		for (unsigned int i=0;i<4;i++)
			children[i]->gatherForces(p);
	}else{
		iterate=data.getBegin();
		for(;iterate!=data.getEnd();iterate++){
			distance2=(iterate->x -p.x)*(iterate->x -p.x) + (iterate->y -p.y)*(iterate->y-p.y);
			distance=sqrt(distance2);
			if (distance>0.001){
				p.fx+=(mass*(cm_x-p.x)/(distance2*distance));
				p.fy+=(mass*(cm_y-p.y)/(distance2*distance));
			}
		}
	}
}

void NBodySimulation::assignForces(NBodySimulation* root){
	if (isLeafNode()){
		list<Point>::iterator iterate=data.getBegin();
		for(;iterate!=data.getEnd();iterate++){
			root->gatherForces(iterate);
		}
	}else{
		for (unsigned int i=0;i<4;i++)
			children[i]->assignForces(root);
	}
}

void NBodySimulation::moveObjects(){
	if (isLeafNode()){
		list<Point>::iterator iterate=data.getBegin();
		for(;iterate!=data.getEnd();iterate++){
			iterate->vx+=iterate->fx;
			iterate->vy+=iterate->fy;

			iterate->x+=iterate->vx;
			iterate->y+=iterate->vy;
		}
	}else{
		for (unsigned int i=0;i<4;i++)
			children[i]->moveObjects();
	}
}

void NBodySimulation::reorder(NBodySimulation* root){
	if (isLeafNode()){
		list<Point>::iterator iterate=data.getBegin();
		for(;iterate!=data.getEnd();iterate++){
			if (!boundsTest(*iterate)){
				root->addPoint(*iterate);
				data.removePoint(iterate);
			}
		}
	}else{
		for (unsigned int i=0;i<4;i++)
			children[i]->reorder(root);
	}
}

void NBodySimulation::colasce(){//combine children
	if (!isLeafNode() && mass < maxPoints){
		collectPoints(data);
		for (int i=0;i<4;i++){
			children[i]->deleteSelf(true);
		}
		children.clear();
	}else{
		for (int i=0;i<4;i++){
			children[i]->colasce();
		}
	}
}



NBodySimulation::NBodySimulation() {
	mass=0;
	cm_x=cm_y=0;
}

NBodySimulation::~NBodySimulation() {
	// TODO Auto-generated destructor stub
}
