/*
 * QuadTree.h
 *
 *  Created on: Feb 5, 2009
 *      Author: sri
 */

#ifndef QUADTREE_H_
#define QUADTREE_H_

#include "PointList.h"
#include <stdexcept>
#include "VectorLine.h"
#include <climits>//FIXME: not needed?
#include <vector>

#define SIMULATION_ACCURACY 1


class QuadTree{
protected:
	vector<QuadTree *> children;
	QuadTree * parent;
	unsigned int currentDepth;
	unsigned char quadrant;//FIXME: quadrant may not be needed...

	float x1,x2,y1,y2;//x and y bounds of this node
	QuadTree* leftWallOwner;
	QuadTree* rightWallOwner;
	QuadTree* topWallOwner;
	QuadTree* bottomWallOwner;

	int mass;
	float cm_x,cm_y;//center of mass

	PointList data;
	static unsigned int maxDepth;//maximum depth of tree
	static unsigned int maxPoints;//max number of points in a leaf. If maxDepth is reached, maxPoints is ignored.

	//These 4 denote the quadrant masks.
	//Note that these are NOT quadrant numbers. They are more like sets.
	//The combination of these masks can denote anywhere between
	//none and all of the quadrants.
	static unsigned char Q_ONE;
	static unsigned char Q_TWO;
	static unsigned char Q_THREE;
	static unsigned char Q_FOUR;

	//internal functions
	void setQuadrant(unsigned char quadrant);
	void propogateBounds();
	bool boundsTest(const Point& p) const;
	void splitNode();
	void propogateData();
	void recombine();

	void collectChildren(vector<QuadTree*>& storage, unsigned char quadrants, unsigned int minDepth=0, unsigned int maxDepth=UINT_MAX);

	float hitTest(const VectorLine& v, float threshold);//for ray tracing
public:

	//basic tree functionality
	QuadTree();
	QuadTree* getParent();
	QuadTree* getChild(int index);
	bool addChild(QuadTree* child);
	void deleteSelf(bool recursive=true);

	static void setMaxDepth(unsigned int d);
	static void setMaxPointsPerLeaf(unsigned int p);

	//functions to be exposed
	QuadTree* addPoint(const Point& p);
	PointList* getPointList();
	unsigned int getNumPoints();

	static unsigned int getMaxDepth();
	static unsigned int getMaxPointsPerLeaf();

	bool isLeafNode();
	void setBounds(float x1, float x2, float y1, float y2);

	void collectPoints(PointList& p);

	float getX1();
	float getX2();
	float getY1();
	float getY2();

	float getCenterX();
	float getCenterY();

	//query functions
	QuadTree* traverseTree(const Point& p, unsigned int levels=UINT_MAX);
	vector<QuadTree*> getLeftNeighbours(unsigned int far_depth=0);
	vector<QuadTree*> getRightNeighbours(unsigned int far_depth=0);
	vector<QuadTree*> getTopNeighbours(unsigned int far_depth=0);
	vector<QuadTree*> getBottomNeighbours(unsigned int far_depth=0);

	vector<QuadTree*> getNE_Neighbour(unsigned int far_depth=0);
	vector<QuadTree*> getNW_Neighbour(unsigned int far_depth=0);
	vector<QuadTree*> getSE_Neighbour(unsigned int far_depth=0);
	vector<QuadTree*> getSW_Neighbour(unsigned int far_depth=0);

	vector<QuadTree*> getNearNeighbours();
	vector<QuadTree*> getFarNeighbours();//returns far neighbors (may include near neighbors)

	QuadTree * rayTrace(const VectorLine& v, float threshold, float* distance=NULL, float corner=0);
	vector<QuadTree*> drawLine(const VectorLine& v, float threshold, float corner);


	void calculateCM();
	void gatherForces(Point& p);
	void assignForces(QuadTree* root);
	void moveObjects();
	void reorder(QuadTree* root);
	void colasce();

};


#endif /* QUADTREE_H_ */
