/*
 * VectorLine.cpp
 *
 *  Created on: Feb 9, 2009
 *      Author: sri
 */

#include "VectorLine.h"
#include <limits>
#include <cmath>

using namespace std;

VectorLine::VectorLine(){
	px=py=0;
	dx=dy=0;
}

VectorLine::VectorLine(const VectorLine& v){//dx=x2-x1, dy=y2-y1
	px=v.px;
	py=v.py;
	dx=v.dx;
	dy=v.dy;
}

VectorLine::VectorLine(float x1,float y1,float x2,float y2){//dx=x2-x1, dy=y2-y1
	setLine(x1,y1,x2,y2);
}

void VectorLine::setLine(float x1,float y1,float x2,float y2){//dx=x2-x1, dy=y2-y1
	px=x1;
	py=y1;
	dx=x2-x1;
	dy=y2-y1;
	
	float normalize=sqrt(dx*dx+dy*dy);
	dx=dx/normalize;
	dy=dy/normalize;
}

void VectorLine::setDirection(float dx,float dy){
	float normalize=sqrt(dx*dx+dy*dy);
	this->dx=dx/normalize;
	this->dy=dy/normalize;
}

float VectorLine::hIntercept(float yval) const{//intercept with horizontal line
	if(dy==0){
		return numeric_limits<float>::max();
	}
	return (yval-py)/dy;
}

float VectorLine::vIntercept(float xval) const{//intercept with vertical line
	if(dx==0){
		return numeric_limits<float>::max();
	}
	return (xval-px)/dx;
}

float VectorLine::evaluateX(float t) const{
	return px+t*dx;
}

float VectorLine::evaluateY(float t) const{
	return py+t*dy;
}

//gives perpendicular distance of vector from point (x,y)
float VectorLine::perpendicularDistance(float x, float y) const{
	float qx=x-px, qy=y-py;
	float qLen_2=(qx*qx)+(qy*qy);
	float qDot=qx*dx+qy*dy;
	return sqrt(qLen_2-qDot*qDot);
}

float VectorLine::delta=0.0001;
