//============================================================================
// Name        : input.cpp
// Author      : Sriram Kashyap
//============================================================================

#include <cstdlib>
#include <iostream>
#include <vector>
#include "VectorLine.h"
#include "OrthoCamera.h"
#include "QuadTree.h"
#include "input.h"
#include "render.h"

using namespace std;

extern OrthoCamera cam;
extern QuadTree tree;
extern QuadTree* selectedTree;
extern QuadTree* rayTarget;
extern void renderScene();
extern void initTree(char * fname);
extern void saveQuadTree(QuadTree* tree, char * fname);

extern bool showNear, showFar, showRayTrace, showRayLine;
extern vector<QuadTree *> rayLine;
extern char* dataFile;

int oldX=0, oldY=0;//variables for dragging
bool dragging=false;
bool insertMode=false;
float rayDirectionX=1, rayDirectionY=0;


void resetEverything(){
	tree.deleteSelf(true);
	selectedTree=NULL;
	rayTarget=NULL;
	showRayTrace=false, showRayLine=false;
	rayLine.clear();
	dragging=false;
	insertMode=false;
	oldX=oldY=0;
	rayDirectionX=rayDirectionY=0;	

	cam.resetCamera();
	cam.changeSize((int)cam.getWidth(),(int)cam.getHeight());
	initTree(dataFile);
}

void processNormalKeys(unsigned char key, int x, int y) {
	static VectorLine v;
	static Point p;
	//int shift= glutGetModifiers();
	if (key == 27){
		exit(0);
	}else if (key=='i' || key=='I'){
		cam.zoomIn();
	}else if (key=='o' || key=='O'){
		cam.zoomOut();
	}else if (key=='-' && selectedTree!=NULL){
		cam.setFocus(selectedTree->getCenterX(),selectedTree->getCenterY());		
		cam.zoomOut();
		cam.resetFocus();
	}else if (key=='=' && selectedTree!=NULL){
		cam.setFocus(selectedTree->getCenterX(),selectedTree->getCenterY());		
		cam.zoomIn();
		cam.resetFocus();
	}else if (key=='q' || key=='Q'){
		rayDirectionX=-1.01;	rayDirectionY=-1;
		showRayTrace=true; showNear=false; showFar=false;
	}else if (key=='w' || key=='W'){
		rayDirectionX=0;	rayDirectionY=-1;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='e' || key=='E'){
		rayDirectionX=1.01;	rayDirectionY=-1;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='d' || key=='D'){
		rayDirectionX=1;	rayDirectionY=0;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='c' || key=='C'){
		rayDirectionX=1.01;	rayDirectionY=1;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='x' || key=='X'){
		rayDirectionX=0;	rayDirectionY=1;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='z' || key=='Z'){
		rayDirectionX=-1.01;	rayDirectionY=1;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='a' || key=='A'){
		rayDirectionX=-1;	rayDirectionY=0;
		showRayTrace=true; showNear=false; showFar=false;		
	}else if (key=='s' || key=='S'){
		showRayTrace=!showRayTrace;
	}else if (key=='n' || key=='N'){
		showNear=!showNear;
	}else if (key=='f' || key=='F'){
		showFar=!showFar;
	}else if (key=='l' || key=='L'){
		showRayLine=!showRayLine;
	}else if (key=='r' || key=='R'){
		resetEverything();
	}else if (key=='T'){
		insertMode=!insertMode;
	}
	if (selectedTree){
		v.setLine(selectedTree->getCenterX(),selectedTree->getCenterY(),
					selectedTree->getCenterX()+rayDirectionX,selectedTree->getCenterY()+rayDirectionY);
		rayTarget=selectedTree->rayTrace(v,0,NULL,0.1);
	}
	renderScene();		
	//GLUT_ACTIVE_SHIFT
	//GLUT_ACTIVE_CTRL
	//GLUT_ACTIVE_ALT

}

void processSpecialKeys(int key, int x, int y) {
	if (key==GLUT_KEY_F5){//save to file
		saveQuadTree(&tree,"temp.txt");
	}
//	GLUT_KEY_F1		F1 function key
//	GLUT_KEY_LEFT		Left function key
//	GLUT_KEY_RIGHT		Up function key
//	GLUT_KEY_UP		Right function key
//	GLUT_KEY_DOWN		Down function key
//	GLUT_KEY_PAGE_UP	Page Up function key
//	GLUT_KEY_PAGE_DOWN	Page Down function key
//	GLUT_KEY_HOME		Home function key
//	GLUT_KEY_END		End function key
//	GLUT_KEY_INSERT		Insert function key
}

void upNormalKeys(unsigned char key,int x,int y){

}
void upSpecialKeys(int key,int x, int y){

}

void mouseWheel(int button, int dir, int x, int y)
{
	if (dir > 0){
		cam.zoomIn();
	}else{
		cam.zoomOut();
	}
	renderScene();
	return;
}

void activeMouseMove(int x, int y){

	if (dragging==true){
		cam.pan(x-oldX,y-oldY);
		oldX=x;oldY=y;
		//std::cout<<"Helloworld";
		renderScene();
	}
}

void passiveMouseMove(int x, int y){
	static VectorLine v;
	if (showRayLine==true){
		rayLine.clear();
		if (selectedTree!=NULL){
			v.setLine(selectedTree->getCenterX(),selectedTree->getCenterY(),cam.iTranX(x),cam.iTranY(y));			
			rayLine=selectedTree->drawLine(v,0,0);
		}
	}
	renderScene();
}
void mouseClick(int button, int state, int x, int y){

	if (button==GLUT_LEFT_BUTTON && state==GLUT_UP){
		if (insertMode==true){
			selectedTree=NULL;
			tree.addPoint(Point(cam.iTranX(x),cam.iTranY(y)));
		}else{
			selectPoint(cam.iTranX(x),cam.iTranY(y));
		}
	}

	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN){
		oldX=x;
		oldY=y;
		dragging=true;
	}

	if (button==GLUT_RIGHT_BUTTON && state==GLUT_UP){
		cam.pan(x-oldX,y-oldY);
		dragging=false;
	}

	renderScene();
}
