//============================================================================
// Name        : quadTree.cpp
// Author      : Sriram Kashyap
//============================================================================

#include <iostream>
#include <cstdlib>
#include <fstream>
#include <cmath>//FIXME:not needed?
#include <GL/glut.h>
#include <GL/freeglut_ext.h>

#include "OrthoCamera.h"
#include "QuadTree.h"
#include "NBodySimulation.h"
#include "render.h"
#include "input.h"

using namespace std;

void openGL_Initialize(int argc, char ** argv);
void changeSize(int width, int height);
void initTree(char * fname);
bool readData(QuadTree *tree, char* fname);
void saveQuadTree(QuadTree* tree, char * fname);

QuadTree tree;//root node of quadtree
OrthoCamera cam;//helper object to pan/zoom
char* dataFile;

int main(int argc, char **argv) {

	openGL_Initialize(argc,argv);
	if (argc==2){
		dataFile=argv[1];
		initTree(argv[1]);
	}else{
		dataFile=NULL;
		initTree(NULL);
	}

	glEnable(GL_DEPTH_TEST);

	glutMainLoop();
	return 0;
}

void changeSize(int width, int height){
	cam.changeSize(width,height);
	glViewport(0, 0, (int)cam.getWidth(), (int)cam.getHeight());
	cam.setOrthographicProjection();
}

void simulateAll(){
//	tree.calculateCM();
//	tree.colasce();
//	tree.assignForces(&tree);
//	tree.moveObjects();
//	tree.reorder(&tree);
//	sleep(1);
	renderScene();
}

void openGL_Initialize(int argc, char ** argv){
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowPosition(50,50);
	glutInitWindowSize(640,640);
	glutCreateWindow("Quadtree test");

	glutDisplayFunc(renderScene);
	glutIdleFunc(simulateAll);
	glutReshapeFunc(changeSize);
	//glutIgnoreKeyRepeat(1);

	glutKeyboardFunc(processNormalKeys);
	glutSpecialFunc(processSpecialKeys);
	glutKeyboardUpFunc(upNormalKeys);
	glutSpecialUpFunc(upSpecialKeys);
	glutMouseFunc(mouseClick);
	glutMotionFunc(activeMouseMove);
	glutMouseWheelFunc(mouseWheel);
	glutPassiveMotionFunc(passiveMouseMove);
}

void initTree(char *fname){
	if (fname==NULL){
		tree.setBounds(0,10,0,10);
		for (int i=0;i<2500;i++){
			tree.addPoint(Point((float)rand()*10/RAND_MAX,(float)rand()*10/RAND_MAX));
		}
	}else{
		readData(&tree,fname);
	}
	cam.setTreeCenter(tree.getCenterX(),tree.getCenterY());
	cam.resetCamera();
}

bool readData(QuadTree * tree, char * fname){
	ifstream infile(fname);
	if (!infile.is_open()){
		return false;
	}

	unsigned int i,n;
	float x,y;
	float minX, minY, maxX, maxY;
	infile>>i;
	tree->setMaxPointsPerLeaf(i);
	infile>>n;//number of points

	//assume there is atleast one data point
	infile>>x>>y;
	minX=maxX=x;
	minY=maxY=y;
	unsigned int cnt=0;
	while (!infile.eof() && cnt<n){
		infile>>x>>y;
		minX=(x<minX)?x:minX;
		minY=(y<minY)?y:minY;
		maxX=(x>maxX)?x:maxX;
		maxY=(y>maxY)?y:maxY;
		cnt++;
	}
	infile.close();

	tree->setBounds(minX-0.01,maxX+0.01,minY-0.01,maxY+0.01);

	//open file again to read data.
	ifstream infile2(fname);
	if (!infile2.is_open()){
		return false;
	}

	infile2>>i>>n;//read first 2 numbers
	Point p;
	cnt=0;
	while (!infile2.eof() && cnt<n){
		infile2>>x>>y;
		p.x=x;
		p.y=y;
		tree->addPoint(p);
		cnt++;
	}
	infile2.close();

	return true;
}

void saveQuadTree(QuadTree* tree, char * fname){
	//open file again to read data.
	ofstream outfile(fname);
	if (!outfile.is_open()){
		return;
	}
	PointList p;//get a list of all points
	list<Point>::iterator iterate;

	tree->collectPoints(p);

	outfile<<QuadTree::getMaxPointsPerLeaf()<<endl<<p.getNumPoints()<<endl;

	iterate=p.getBegin();
	for(;iterate!=p.getEnd();iterate++){
		outfile<<iterate->x<< " "<<iterate->y<<endl;
	}
	outfile.close();
}
