#include "render.h"
#include "VectorLine.h"
#include "GL/glut.h"

extern QuadTree tree;
extern OrthoCamera cam;

vector<QuadTree *> nearCells;
vector<QuadTree *> farCells;
vector<QuadTree *> rayLine;

bool showNear=false;
bool showFar=false;
bool showRayTrace=false;
bool showRayLine=false;

QuadTree * selectedTree;
QuadTree * rayTarget;

extern float rayDirectionX, rayDirectionY;
extern void simulateAll();

#define BRIGHT 170

unsigned char GREEN[3]={0,BRIGHT,0};
unsigned char RED[3]={BRIGHT,0,0};
unsigned char BLUE[3]={0,0,BRIGHT};
unsigned char YELLOW[3]={BRIGHT,BRIGHT,0};
unsigned char BLACK[3]={0,0,0};
unsigned char WHITE[3]={255,255,255};
unsigned char GRAY[3]={BRIGHT,BRIGHT,BRIGHT};
unsigned char CYAN[3]={0,BRIGHT,BRIGHT};


void renderTree(QuadTree* q, unsigned char* color, bool recursive){
	static float x1,y1,x2,y2;
	static PointList* p;
	static list<Point>::iterator iterate;

	if (q->isLeafNode()){
		x1=cam.tranX(q->getX1());
		x2=cam.tranX(q->getX2());
		y1=cam.tranY(q->getY1());
		y2=cam.tranY(q->getY2());

		if (color!=NULL){
			glBegin(GL_QUADS);
				glColor3ubv (color);
				glVertex3f(x1,y1,0);
				glVertex3f(x1,y2,0);
				glVertex3f(x2,y2,0);
				glVertex3f(x2,y1,0);
			glEnd();
		}

		glBegin(GL_LINE_LOOP);
			glColor3ubv (WHITE);//border
			glVertex3f(x1,y1,.5);
			glVertex3f(x1,y2,.5);
			glVertex3f(x2,y2,.5);
			glVertex3f(x2,y1,.5);
		glEnd();

		glPointSize(2.0f);
		glBegin(GL_POINTS);
			glColor3ubv (WHITE);
			p=q->getPointList();
			iterate=p->getBegin();

			for (;iterate!=p->getEnd();iterate++){
				glVertex3f(	cam.tranX(iterate->x),
							cam.tranY(iterate->y),.5);
			}
		glEnd();
		glPointSize(1.0f);
	}else if (recursive){
		for (int i=0;i<4;i++){
			renderTree(q->getChild(i),color,true);
		}
	}
}

void renderTestPoints(){
	glPointSize(10.0f);
	glBegin(GL_QUADS);
		glColor3ub (250,0,0);
		glVertex3f(0,0,0);
		glColor3ub (0,250,0);
		glVertex3f(0,cam.getHeight(),0);
		glColor3ub (0,0,250);
		glVertex3f(cam.getWidth(),cam.getHeight(),0);
		glColor3ub (250,250,0);
		glVertex3f(cam.getWidth(),0,0);
	glEnd();
	glPointSize(1.0f);
}

void renderScene(void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	renderTree(&tree,NULL);

	if (selectedTree){
		renderTree(selectedTree,RED,false);

		if (showNear){
			for (unsigned int i=0;i<nearCells.size();i++){
				if (nearCells[i]->getNumPoints()==0)
					renderTree(nearCells[i],BLUE,false);
				else
					renderTree(nearCells[i],GREEN,false);
			}
		}
		if (showFar){
			for (unsigned int i=0;i<farCells.size();i++){
				//note that farcells can have null elements
				if (farCells[i]!=NULL){
					if (farCells[i]->getNumPoints()==0)
						renderTree(farCells[i],BLUE,false);
					else
						renderTree(farCells[i],GREEN,false);
				}
			}
		}
		if (showRayTrace && rayTarget){
			renderTree(rayTarget,YELLOW,false);
		}
		if (showRayLine){
			for (unsigned int i=0;i<rayLine.size();i++){
				renderTree(rayLine[i],GREEN,false);
			}
		}
	}
	//renderTestPoints();

	glPopMatrix();
	glutSwapBuffers();

	//simulateAll();
}

//if an element occurs both in v1 and v2, set it to null in v1
void subtractVectors(vector<QuadTree*>& v1, vector<QuadTree*>& v2){
	for (unsigned int i=0;i<v1.size();i++){
		for (unsigned int j=0;j<v2.size();j++){
			if (v1[i]==v2[j]){
				v1[i]=NULL;
			}
		}
	}
}

void selectPoint(float x, float y){
	static vector<QuadTree*> emptyVector;
	static VectorLine v;

	selectedTree=tree.traverseTree(Point(x,y));
	nearCells.clear();
	farCells.clear();
	if (selectedTree){
		nearCells=selectedTree->getNearNeighbours();
		farCells=selectedTree->getFarNeighbours();
		subtractVectors(farCells,nearCells);

		v.setLine(selectedTree->getCenterX(),selectedTree->getCenterY(),
				selectedTree->getCenterX()+rayDirectionX,selectedTree->getCenterY()+rayDirectionY);
		rayTarget=selectedTree->rayTrace(v,0,NULL,0.1);
	}else{
		rayTarget=NULL;
	}

}
